var Istat = function()
{
	var istat = {};
	var debug = 0;
	
	var incaFileIsValid = false;

	var REPORT_TYPE_STATISTICS = 'statistics';
	var REPORT_TYPE_STATISTICS_CSV = 'statistics_csv';
	var incaFilesAreValid = false;
	var incaFileStructuresAreValid = false;
	var jsonIncaData = {};

	istat.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', istat.validateStep1);
		Events.addEventHandler('startDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('endDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('incafile', 'change', istat.incaFileOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', istat.initializeStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('simplestatistics', istat.startCheck);
		Buttons.addClickHandler('fullstatistics', istat.startCheck);
		Buttons.addClickHandler('downloadreportfile', istat.downloadReportFile);
		Buttons.addClickHandler('downloadoutputfile', istat.downloadReportFile);

		Doc.show('step-1');
		Doc.show('simplestatistics');
		Doc.show('fullstatistics');
		
		Doc.element('statisticsDateVar').focus();
		Wizard.setInputStatus('incafile', 'invalid', 'Please select a file!');
		istat.incaFileOnChange();
	}
	
	istat.startCheck = function(event)
	{
		checkType = event.currentTarget.id;
		Doc.hide('stat-table');
		Doc.hide('downloadreportfile');
		Wizard.setCurrentStep(1);
		Doc.addClass('step-2', 'disabled');
		Wizard.goNext();
	}
	
	istat.validateStep1 = function(event)
	{
		var valid = incaFileIsValid;
		valid &= Wizard.isValid('startDate');
		valid &= Wizard.isValid('endDate');
		if (!valid) event.preventDefault();
	}

	istat.incaFileOnChange = function()
	{
		incaFileIsValid = false;
		Wizard.setInputStatus('incafile');
		
		var frm = document.forms['reggie'];
		var files = frm.incafile.files;

		if (files.length == 1)
		{
			incaFileIsValid = true;
			Wizard.setInputStatus('incafile', 'valid');
		}
		else
		{
			Wizard.setInputStatus('incafile', 'invalid', 'A file must be selected.');
		}
	}


	istat.initializeStep2 = function()
	{
		Doc.hide('explainFileCheck');
		var frm = document.forms['reggie'];
		var statDateVar = frm.statisticsDateVar.value;
		var startDate = frm.startDate.value;
		var endDate = frm.endDate.value;
		var cancerType = frm.cancerType.value;

		var formData = new FormData();
		var files = frm.incafile.files;
		for (var i = 0; i < files.length; i++)
		{
			var file = files[i];
			formData.append('incafiles[]', file, file.name);
		}

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=IncaStatistics';
		url += '&statdatevar='+encodeURIComponent(statDateVar);
		url += '&startdate='+encodeURIComponent(startDate);
		url += '&enddate='+encodeURIComponent(endDate);
		url += '&cancertype='+encodeURIComponent(cancerType);
		url += '&checkType=' + checkType;

		// POST
		Wizard.showLoadingAnimation('Checking INCA data...', 'inca-stat-progress');
		Wizard.asyncJsonRequest(url, istat.checkCompleted, 'POST', formData);
	}

	istat.checkCompleted = function(response)
	{
		var frm = document.forms['reggie'];

		var fileInfo = response.fileInfo;
		var headers = fileInfo.headers;
		var checkType = response.checkType;
		var fileCanBeUsed = true;

		Doc.element('filename').innerHTML = Strings.encodeTags(fileInfo.filename);
		Doc.element('isFollowUp').innerHTML = fileInfo.isFollowUp ? 'Yes' : 'No';
		if (fileInfo.isFollowUp)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('isFollowUp', 'invalid', 'A follow-up file can\'t be used for statistics.');
		}
		Doc.element('headerCount').innerHTML = headers.count;
		
		Doc.element('headerMissing').innerHTML = headers.missing;
		if (headers.missing > 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('headerMissing', 'invalid', 'One ore more required headers are missing: <b>' + Strings.encodeTags(headers.missingNames.join(', ')) + '</b>');
		}
		else
		{
			Wizard.setInputStatus('headerMissing', 'valid');
		}
		
		Doc.element('headerMissingAnnotationTypes').innerHTML = headers.missingAnnotationTypes;
		if (headers.missingAnnotationTypes > 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('headerMissingAnnotationTypes', 'invalid', 'One ore more required INCA annotations are missing in BASE: <b>' + Strings.encodeTags(headers.missingAnnotationTypeNames.join(', ')) + '</b>');
		}
		else
		{
			Wizard.setInputStatus('headerMissingAnnotationTypes', 'valid');
		}
		
		Doc.element('headerDuplicates').innerHTML = headers.duplicates;
		if (headers.duplicates > 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('headerDuplicates', 'invalid', 'Duplicate headers must be removed before the file can be imported: <b>' + Strings.encodeTags(headers.duplicateNames.join(', ')) + '</b>');
		}
		else
		{
			Wizard.setInputStatus('headerDuplicates', 'valid');
		}
		
		var statisticsLineCount = fileInfo.data.count - fileInfo.data.totalExcluded;
		Doc.element('lineCount').innerHTML = fileInfo.data.count;
		Doc.element('statisticsLineCount').innerHTML = statisticsLineCount;
		Doc.element('totalExcluded').innerHTML = fileInfo.data.totalExcluded;
		if (fileInfo.data.count == 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('lineCount', 'invalid', 'The file contains no data.');
		}
		else if (statisticsLineCount == 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('statisticsLineCount', 'invalid', 'The file contains no data that can be used for statistics.');
		}
		else 
		{
			Wizard.setInputStatus('statisticsLineCount', 'valid');
		}
		
		if (istat.setExcludedLines('tooManyColumns', fileInfo.data.TOO_MANY_COLUMNS) > 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('tooManyColumns', 'invalid', 'All data lines must have the same number of columns.');
		}
		
		if (istat.setExcludedLines('tooFewColumns', fileInfo.data.TOO_FEW_COLUMNS) > 0)
		{
			fileCanBeUsed = false;
			Wizard.setInputStatus('tooFewColumns', 'invalid', 'All data lines must have the same number of columns.');
		}
		
		if (istat.setExcludedLines('missingLaterality', fileInfo.data.MISSING_LATERALITY) > 0)
		{
			Wizard.setInputStatus('missingLaterality', 'warning', 'Lines without laterality are skipped.');
		}
		
		if (istat.setExcludedLines('duplicateLaterality', fileInfo.data.DUPLICATE_LATERALITY) > 0)
		{
			Wizard.setInputStatus('duplicateLaterality', 'warning', 'Lines with same personal number/id and laterality are skipped.');
		}

		if (istat.setExcludedLines('invalidDataValue', fileInfo.data.INVALID_DATA_VALUE) > 0)
		{
			Wizard.setInputStatus('invalidDataValue', 'warning', 'Lines with invalid values are skipped.');
		}
		
		if (istat.setExcludedLines('filteredByDate', fileInfo.data.FILTERED_BY_DATE) > 0)
		{
			var opt = frm.statisticsDateVar[frm.statisticsDateVar.selectedIndex];
			Wizard.setInputStatus('filteredByDate', 'valid', 'Only lines with <b>' + opt.text + '</b> between <b>' + Reggie.reformatDate(frm.startDate.value) + '</b> and <b>' + Reggie.reformatDate(frm.endDate.value) + '</b> are included.');
		}
		
		if (istat.setExcludedLines('filteredByCancerType', fileInfo.data.FILTERED_BY_CANCER_TYPE) > 0)
		{
			var opt = frm.cancerType[frm.cancerType.selectedIndex];
			Wizard.setInputStatus('filteredByCancerType', 'valid', 'Only lines with <b>'+opt.text+'</b> are included.');
		}
		
		if (fileCanBeUsed)
		{
			var statistics = fileInfo.statistics;
			var statHtml = '';
			for (var i = 0; i < statistics.length; i++)
			{
				var statVar = statistics[i];
				var notNa = statVar.notNa;
				for (var g = 0; g < statVar.groups.length; g++)
				{
					var grp = statVar.groups[g];
					var noPct = grp.isNa || !statVar.hasNaGroup;

					if (g == 0)
					{
						statHtml += '<tr class="stat-var">';
						statHtml += '<td class="stat-var-name">'+Strings.encodeTags(statVar.name)+'</td>';
					}
					else
					{
						statHtml += '<tr>';
						if (g == 1 && statVar.unit)
						{
							statHtml += '<td class="stat-var-unit">('+statVar.unit+')</td>';
						}
						else
						{
							statHtml += '<td></td>';
						}
					}
					statHtml += '<td class="stat-grp-name">'+Strings.encodeTags(grp.name)+'</td>';
					statHtml += '<td class="count">'+grp.total+'</td>';
					statHtml += '<td class="percentage">'+(noPct ? '' : Reggie.formatNumber(100*grp.total/notNa.total, '%', 1))+'</td>';
					statHtml += '<td class="fullstatistics count">'+grp.accruedSpecimen+'</td>';
					statHtml += '<td class="fullstatistics percentage">'+(noPct ? '' : Reggie.formatNumber(100*grp.accruedSpecimen/notNa.accruedSpecimen, '%', 1))+'</td>';
					statHtml += '<td class="count">'+grp.accruedNoSpecimen+'</td>';
					statHtml += '<td class="percentage">'+(noPct ? '' : Reggie.formatNumber(100*grp.accruedNoSpecimen/notNa.accruedNoSpecimen, '%', 1))+'</td>';
					statHtml += '<td class="count">'+grp.notAccrued+'</td>';
					statHtml += '<td class="percentage">'+(noPct ? '' : Reggie.formatNumber(100*grp.notAccrued/notNa.notAccrued, '%', 1))+'</td>';
					statHtml += '</tr>';
				}
			}
			Doc.element('stat-data').innerHTML = statHtml;
			Doc.show('stat-table', 'table');
			Data.set('stat-table', 'check-type', checkType);
		}
		
		Wizard.setCurrentStep(2);
		if (fileCanBeUsed)
		{
			if (checkType == 'simplestatistics')
			{
				Doc.show('fullstatistics');
			}
			else
			{
				Doc.show('downloadoutputfile');
			}
		}
		else
		{
			Wizard.setWizardStatus('messagecontainer error', 'Statistics is not possible.');
		}
		
		Wizard.setNoConfirm(true);
		Doc.show('gorestart');
		Doc.show('downloadreportfile');
	}

	/**
		Set the value for an 'excluded line' variable. If the
		number of excluded lines is 0 the entire row is hidden.
	*/
	istat.setExcludedLines = function(elementId, numLines)
	{
		Doc.element(elementId).innerHTML = numLines;
		Doc.showHide(elementId+'.row', numLines > 0);
		return numLines;
	}

	istat.downloadReportFile = function(event)
	{
		var frm = document.forms['reggie'];
		var reportType = Data.get(event.currentTarget, 'report-type');

		var url = '../Inca.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadIncaReportFile';
		url += '&reporttype='+encodeURIComponent(reportType);
		window.open(url);
	}

	return istat;
}();

Doc.onLoad(Istat.initPage);

